package de.mdelab.sdm.interpreter.core.executionTrace;

import org.eclipse.emf.ecore.EObject;

import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.SDMInterpreter;
import de.mdelab.sdm.interpreter.core.notifications.ActivityNodeExecutionFinishedNotification;
import de.mdelab.sdm.interpreter.core.notifications.ActivityNodeExecutionStartedNotification;
import de.mdelab.sdm.interpreter.core.notifications.InterpreterNotification;
import de.mdelab.sdm.interpreter.core.notifications.NotificationReceiver;
import de.mdelab.sdm.interpreter.core.notifications.NotificationTypeEnum;
import de.mdelab.sdm.interpreter.core.variables.NotifierVariablesScope;

public abstract class PerformanceNotificationReceiver<StoryDiagramElement extends EObject, ActivityType extends EObject, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType>
		implements NotificationReceiver<ClassifierType>, ISDMExecutionProfile {
	
	protected static final NotificationTypeEnum[] DEFAULT_LISTENED_NOTIFICATION_TYPES = new NotificationTypeEnum[]{ NotificationTypeEnum.ACTIVITY_NODE_EXECUTION_STARTED,
																													NotificationTypeEnum.STORY_PATTERN_INITIALIZATION_STARTED,
																													NotificationTypeEnum.STORY_PATTERN_MATCHING_STARTED,
																													NotificationTypeEnum.STORY_PATTERN_CONSTRAINT_HOLDS,
																													NotificationTypeEnum.STORY_PATTERN_CONSTRAINT_VIOLATED,
																													NotificationTypeEnum.STORY_PATTERN_OBJECT_BOUND,
																													NotificationTypeEnum.STORY_PATTERN_OBJECT_NOT_BOUND,
																													NotificationTypeEnum.STORY_PATTERN_OBJECT_BINDING_REVOKED,
																													NotificationTypeEnum.EVALUATED_EXPRESSION,
																													NotificationTypeEnum.TRAVERSING_LINK };

	
	private Long maxActivityNodeExecTime = 1500l;
	
	private long activityNodeExecutionStartTime = 0;

	private final ExecutionTraceNotificationReceiver<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> execTraceReceiver;
	
	private NotifierVariablesScope<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> initialVariableScope;
	
	private boolean active;

	public PerformanceNotificationReceiver() {
		execTraceReceiver = createExecutionTraceReceiver();
		initialVariableScope = null;
		active = true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void notifyChanged( final InterpreterNotification<ClassifierType> p_notification ) {
		if ( isActive() ) {
			switch ( p_notification.getNotificationType() )	{
				case ACTIVITY_NODE_EXECUTION_STARTED: {
					handleActivityNodeExecutionStarted( (ActivityNodeExecutionStartedNotification<ActivityNodeType, ClassifierType>) p_notification );
	
					break;
				}
				
				case ACTIVITY_NODE_EXECUTION_FINISHED:	{
					handleActivityNodeExecutionFinished( (ActivityNodeExecutionFinishedNotification<ActivityNodeType, ClassifierType>) p_notification );
	
					break;
				}
				
				default:
					break;
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void handleActivityNodeExecutionStarted( final ActivityNodeExecutionStartedNotification<ActivityNodeType, ClassifierType> p_notification ) {
		activityNodeExecutionStartTime = System.currentTimeMillis();
		
		initialVariableScope = (NotifierVariablesScope<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType>) p_notification.getVariablesScope().clone();
	}

	private void handleActivityNodeExecutionFinished( final ActivityNodeExecutionFinishedNotification<ActivityNodeType, ClassifierType> p_notification ) {
		final long executionTime =  System.currentTimeMillis() - activityNodeExecutionStartTime;
		
		if ( executionTime > maxActivityNodeExecTime ) {
			@SuppressWarnings("unchecked")
			final ActivityNodeType activityNode = (ActivityNodeType) p_notification.getMainStoryDiagramElement();
			@SuppressWarnings("unchecked")
			final SDMInterpreter<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> interpreter = (SDMInterpreter<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType>) p_notification.getNotifier();
			handleActivityNodeExecutionTimeExceeded( interpreter, activityNode,	executionTime );
		}
	}
	
	protected void handleActivityNodeExecutionTimeExceeded( 	final SDMInterpreter<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> p_interpreter,
															final ActivityNodeType p_activityNode,
															final long pl_executionTime ) {
		System.out.println( "********************** Executed activity node " + getText( p_activityNode ) + " of activity " + getText( getActivity( p_activityNode ) ) + " in " + pl_executionTime + "ms." );
		System.out.println( initialVariableScope );
		
		try {
			traceReExecution( p_interpreter, p_activityNode );
		} 
		catch ( final SDMException p_ex) {
			p_ex.printStackTrace();
		}
	}

	protected void traceReExecution( 	final SDMInterpreter<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> p_interpreter,
										final ActivityNodeType p_activityNode ) 
	throws SDMException {
		assert execTraceReceiver != null;
		assert initialVariableScope != null;
		
		deActivate();
		
		p_interpreter.getNotificationEmitter().addNotificationReceiver( execTraceReceiver );
		
		p_interpreter.executeActivityNode( p_activityNode, initialVariableScope );
		
		p_interpreter.getNotificationEmitter().removeNotificationReceiver( execTraceReceiver );
		
		activate();
	}
	
	public ExecutionTrace getExecutionTrace() {
		final ExecutionTrace trace = execTraceReceiver.getExecutionTrace();
		
		return trace.getExecutions().isEmpty() ? null : trace;
	}
	
	protected void activate() {
		active = true;
	}
	
	protected void deActivate() {
		active = false;
	}
	
	protected boolean isActive() {
		return active;
	}

	protected abstract ExecutionTraceNotificationReceiver<ActivityType, ActivityNodeType, ActivityEdgeType, StoryPatternType, StoryPatternObjectType, StoryPatternLinkType, ClassifierType, FeatureType, ExpressionType> createExecutionTraceReceiver();

	protected abstract String getText( Object p_object );
	
	protected abstract ActivityType getActivity( ActivityNodeType p_activityNode );
}
